import 'package:fake_http/fake_http.dart';
import 'package:star_xms/src/interceptor/cache_constants.dart';

/// 网络层拦截器
///
/// 针对CacheSplitInterceptor中对URL进行修改作对应还原
class CacheSplitFixedInterceptor implements Interceptor {
  @override
  Future<Response> intercept(Chain chain) async {
    Request originalRequest = chain.request();
    String cacheSplit =
        originalRequest.url().queryParameter(CacheConstants.QUERY_CACHE_SPLIT);
    if (cacheSplit == null || cacheSplit.isEmpty) {
      return await chain.proceed(originalRequest);
    }
    Response response = await chain.proceed(originalRequest
        .newBuilder()
        .url(originalRequest
            .url()
            .newBuilder()
            .removeAllQueryParameters(CacheConstants.QUERY_CACHE_SPLIT)
            .build())
        .build());
    return response.newBuilder().request(originalRequest).build();
  }
}
