import 'dart:async';

import 'package:fake_http/fake_http.dart';
import 'package:flutter/foundation.dart';

/// 网络层拦截器
class TokenInterceptor implements Interceptor {
  TokenInterceptor(
    AsyncValueGetter<String> token,
  )   : assert(token != null),
        _token = token;

  static const String HEADER_TOKEN = "token";

  AsyncValueGetter<String> _token;

  @override
  Future<Response> intercept(Chain chain) async {
    Request originalRequest = chain.request();
    String token = await _token();
    if (token == null || token.isEmpty) {
      return await chain.proceed(originalRequest);
    }
    return await chain.proceed(originalRequest
        .newBuilder()
        .header(HEADER_TOKEN, Uri.encodeFull(token)) // encode
        .build());
  }
}
