import 'dart:async';

import 'package:flutter/foundation.dart';
import 'package:flutter/widgets.dart';

abstract class XMSLocalizations {
  String get requestError;

  String get serverError;

  String get networkError;

  String get tutorialLoading;

  String get tutorialError;

  String get tutorialEmpty;

  String get tutorialListMore;

  String get tutorialListEnd;

  static XMSLocalizations of(BuildContext context) {
    return Localizations.of<XMSLocalizations>(context, XMSLocalizations);
  }
}

abstract class GlobalXMSLocalizations implements XMSLocalizations {
  GlobalXMSLocalizations(
    this.locale,
  ) : assert(locale != null);

  final Locale locale;

  static const LocalizationsDelegate<XMSLocalizations> delegate =
      _GMLocalizationsDelegate();

  static GlobalXMSLocalizations translation(Locale locale) {
    switch (locale.languageCode) {
      case 'en':
        return _XMSLocalizationsEn(locale);
      case 'zh':
        switch (locale.countryCode) {
          case 'HK':
          case 'TW':
            return _XMSLocalizationsZhT(locale);
        }
    }
    return _XMSLocalizationsBase(locale);
  }
}

class _GMLocalizationsDelegate extends LocalizationsDelegate<XMSLocalizations> {
  const _GMLocalizationsDelegate();

  static const List<String> _supportedLanguages = <String>[
    'en',
    'zh', // Chinese (simplified)
  ];

  static final Map<Locale, Future<XMSLocalizations>> _loadedTranslations =
      <Locale, Future<XMSLocalizations>>{};

  @override
  bool isSupported(Locale locale) {
    return _supportedLanguages.contains(locale.languageCode);
  }

  @override
  Future<XMSLocalizations> load(Locale locale) {
    assert(isSupported(locale));
    return _loadedTranslations.putIfAbsent(locale, () {
      return SynchronousFuture<XMSLocalizations>(
          GlobalXMSLocalizations.translation(locale));
    });
  }

  @override
  bool shouldReload(LocalizationsDelegate<XMSLocalizations> old) {
    return false;
  }
}

class _XMSLocalizationsBase extends GlobalXMSLocalizations {
  _XMSLocalizationsBase(
    Locale locale,
  ) : super(locale);

  @override
  String get requestError => '网络链接错误';

  @override
  String get serverError => '网络链接错误';

  @override
  String get networkError => '网络链接错误';

  @override
  String get tutorialLoading => '加载中...';

  @override
  String get tutorialError => '网络出问题啦，下拉刷新试试～';

  @override
  String get tutorialEmpty => '暂无数据';

  @override
  String get tutorialListMore => '努力加载中...';

  @override
  String get tutorialListEnd => '已经看到最后啦';
}

class _XMSLocalizationsEn extends _XMSLocalizationsBase {
  _XMSLocalizationsEn(
    Locale locale,
  ) : super(locale);
}

class _XMSLocalizationsZhT extends _XMSLocalizationsBase {
  _XMSLocalizationsZhT(
    Locale locale,
  ) : super(locale);
}
