import 'package:fake_http/fake_http.dart';
import 'package:flutter/rendering.dart';
import 'package:meta/meta.dart';
import 'package:star_xms/src/domain/entity.dart';
import 'package:star_xms/src/exception/http_xms_exception.dart';

abstract class BaseModel {}

abstract class BaseView {
  bool get mounted;
}

abstract class BasePresenter<V extends BaseView> {
  BasePresenter(
    this.view,
  );

  @protected
  V view;
  bool _disposeFlag = false;

  @protected
  bool isUnauthorized(Object error) {
    if (error != null && error is HttpXmsException) {
      return error.code() == HttpStatus.unauthorized;
    }
    return false;
  }

  @protected
  DateTime parseServedDate(Response response) {
    return HttpDate.parse(response.header(HttpHeaders.dateHeader));
  }

  @protected
  DateTime parseReceivedDate(Response response) {
    return DateTime.fromMillisecondsSinceEpoch(
        response.receivedResponseAtMillis());
  }

  @protected
  String entityErrorAI(
    Entity entity,
    String serverError,
  ) {
    if (entity != null && entity.msg != null && entity.msg.isNotEmpty) {
      return entity.msg;
    }
    return serverError;
  }

  @protected
  String catchErrorAI(
    Object error,
    String requestError,
    String serverError,
    String networkError,
  ) {
    if (error != null && error is HttpXmsException) {
      HttpXmsException exception = error;
      if (exception.code() >= HttpStatus.internalServerError) {
        return exception.message() != null && exception.message().isNotEmpty
            ? exception.message()
            : serverError;
      }
      if (exception.code() >= HttpStatus.badRequest) {
        return exception.message() != null && exception.message().isNotEmpty
            ? exception.message()
            : requestError;
      }
    }
    return networkError;
  }

  @protected
  bool isMounted() {
    return view != null && view.mounted;
  }

  @protected
  bool isDisposed() {
    return _disposeFlag;
  }

  void dispose() {
    if (!_disposeFlag) {
      _disposeFlag = true;
      view = null;
      onDispose();
    } else {
      throw throwError();
    }
  }

  @protected
  void onDispose() {}

  @protected
  FlutterError throwError() {
    return FlutterError('$runtimeType is disposed');
  }
}
