import 'package:fake_http/fake_http.dart';
import 'package:jaguar_serializer/jaguar_serializer.dart';
import 'package:meta/meta.dart';
import 'package:star_xms/src/_domain/list_entity.dart';
import 'package:star_xms/src/_domain/page_entity.dart';
import 'package:star_xms/src/_domain/result_entity.dart';
import 'package:star_xms/src/domain/entity.dart';
import 'package:star_xms/src/domain/list_entity.dart';
import 'package:star_xms/src/domain/page_entity.dart';
import 'package:star_xms/src/domain/result_entity.dart';
import 'package:star_xms/src/exception/http_xms_exception.dart';

class BaseRepository {
  @protected
  CacheControl cacheControl(bool refresh) {
    return refresh ? CacheControl.FORCE_NETWORK : null;
  }

  @protected
  Future<Entity> entity(Response response) async {
    if (response != null) {
      if (response.isSuccessful()) {
        String source = await response.body().string();
        return source != null && source.isNotEmpty
            ? EntitySerializer()
                .fromMap(json.decode(source) as Map<dynamic, dynamic>)
            : null;
      } else {
        throw HttpXmsException(response);
      }
    } else {
      throw Exception('Response is null!');
    }
  }

  @protected
  Future<ResultEntity<T>> resultEntityAsPass<T>(Response response) async {
    if (response != null) {
      if (response.isSuccessful()) {
        String source = await response.body().string();
        return source != null && source.isNotEmpty
            ? JserResultEntitySerializer()
                .fromMap(json.decode(source) as Map<dynamic, dynamic>)
                .swapAsPass()
            : null;
      } else {
        throw HttpXmsException(response);
      }
    } else {
      throw Exception('Response is null!');
    }
  }

  @protected
  Future<ResultEntity<T>> resultEntityAsMap<T>(
    Response response,
    Serializer<T> serializer,
  ) async {
    if (response != null) {
      if (response.isSuccessful()) {
        String source = await response.body().string();
        return source != null && source.isNotEmpty
            ? JserResultEntitySerializer()
                .fromMap(json.decode(source) as Map<dynamic, dynamic>)
                .swapAsMap(serializer)
            : null;
      } else {
        throw HttpXmsException(response);
      }
    } else {
      throw Exception('Response is null!');
    }
  }

  @protected
  Future<ListEntity<T>> listEntityAsPass<T>(Response response) async {
    if (response != null) {
      if (response.isSuccessful()) {
        String source = await response.body().string();
        return source != null && source.isNotEmpty
            ? JserListEntitySerializer()
                .fromMap(json.decode(source) as Map<dynamic, dynamic>)
                .swapAsPass()
            : null;
      } else {
        throw HttpXmsException(response);
      }
    } else {
      throw Exception('Response is null!');
    }
  }

  @protected
  Future<ListEntity<T>> listEntityAsMap<T>(
    Response response,
    Serializer<T> serializer,
  ) async {
    if (response != null) {
      if (response.isSuccessful()) {
        String source = await response.body().string();
        return source != null && source.isNotEmpty
            ? JserListEntitySerializer()
                .fromMap(json.decode(source) as Map<dynamic, dynamic>)
                .swapAsMap(serializer)
            : null;
      } else {
        throw HttpXmsException(response);
      }
    } else {
      throw Exception('Response is null!');
    }
  }

  @protected
  Future<PageEntity<T>> pageEntityAsPass<T>(Response response) async {
    if (response != null) {
      if (response.isSuccessful()) {
        String source = await response.body().string();
        return source != null && source.isNotEmpty
            ? JserPageEntitySerializer()
                .fromMap(json.decode(source) as Map<dynamic, dynamic>)
                .swapAsPass()
            : null;
      } else {
        throw HttpXmsException(response);
      }
    } else {
      throw Exception('Response is null!');
    }
  }

  @protected
  Future<PageEntity<T>> pageEntityAsMap<T>(
    Response response,
    Serializer<T> serializer,
  ) async {
    if (response != null) {
      if (response.isSuccessful()) {
        String source = await response.body().string();
        return source != null && source.isNotEmpty
            ? JserPageEntitySerializer()
                .fromMap(json.decode(source) as Map<dynamic, dynamic>)
                .swapAsMap(serializer)
            : null;
      } else {
        throw HttpXmsException(response);
      }
    } else {
      throw Exception('Response is null!');
    }
  }
}
