import 'package:fake_http/fake_http.dart';
import 'package:flutter/foundation.dart';
import 'package:meta/meta.dart';
import 'package:star_xms/src/interceptor/cache_split_fixed_interceptor.dart';
import 'package:star_xms/src/interceptor/cache_split_interceptor.dart';
import 'package:star_xms/src/interceptor/sign_interceptor.dart';
import 'package:star_xms/src/interceptor/token_interceptor.dart';
import 'package:star_xms/src/repository/base_repository.dart';

class XmsRepository extends BaseRepository {
  XmsRepository({
    @required this.serverUrl,
    @required String appId,
    @required String appSecret,
    @required AsyncValueGetter<String> token,
    @required AsyncValueGetter<String> uuid,
    @required AsyncValueGetter<String> userAgent,
    @required AsyncValueGetter<Directory> cacheDir,
    @required AsyncValueGetter<bool> connectivity,
    CookieJar cookieJar,
    LoggerLevel loggerLevel = LoggerLevel.BASIC,
    LoggerFactory loggerFactory = LoggerFactory.PLATFORM,
  }) : client = OkHttpClientBuilder()
            .cache(cacheDir != null ? Cache(DiskCache.create(cacheDir)) : null)
            .cookieJar(cookieJar ??
                PersistentCookieJar.persistent(CookiePersistor.MEMORY))
            .addInterceptor(UserAgentInterceptor(userAgent))
            .addInterceptor(OptimizedRequestInterceptor(connectivity))
            .addInterceptor(CacheSplitInterceptor(token))
            .addNetworkInterceptor(CacheSplitFixedInterceptor())
            .addNetworkInterceptor(OptimizedResponseInterceptor())
            .addNetworkInterceptor(TokenInterceptor(token))
            .addNetworkInterceptor(HttpLoggingInterceptor(
                level: loggerLevel, factory: loggerFactory))
            .addNetworkInterceptor(
                SimpleSignInterceptor(appId, appSecret, uuid))
            .build();

  @protected
  final String serverUrl;
  @protected
  final OkHttpClient client;
}
