import 'package:jaguar_serializer/jaguar_serializer.dart';

part 'server.jser.dart';

@GenSerializer()
class ServerSerializer extends Serializer<Server> with _$ServerSerializer {}

class Server {
  const Server(
    this.appId,
    this.appSecret,
    this.baseUrl,
  );

  static const Server PROD = Server(
    'staredu_teacher',
    'abc7456e2d19953eaf86e362baf0f9e8',
    'https://edu.xrjiot.com',
  );
  static final Server DEBUG = PROD.copyWith(
    baseUrl: 'http://192.168.1.12:8080',
  );
  static final Server RAP = PROD.copyWith(
    baseUrl: 'http://rap.xrjiot.cn',
  );

  final String appId;
  final String appSecret;
  final String baseUrl;

  Server copyWith({
    String appId,
    String appSecret,
    String baseUrl,
  }) {
    return Server(
      appId ?? this.appId,
      appSecret ?? this.appSecret,
      baseUrl ?? this.baseUrl,
    );
  }

  bool isPROD() {
    return baseUrl == PROD.baseUrl;
  }

  bool isRAP() {
    return baseUrl == RAP.baseUrl;
  }

  int serverVer() {
    return isRAP() ? 1 : 11;
  }

  String serverUrl() {
    return isRAP() ? '$baseUrl/mockjsdata/13' : '$baseUrl/xrj-edu-web';
  }

  String serverApiUrl() {
    return '${serverUrl()}/api/app/v${serverVer()}';
  }

  String imServerUrl() {
    return isRAP() ? '$baseUrl/mockjsdata/27/im' : '$baseUrl/im';
  }

  String imServerApiUrl() {
    return '${imServerUrl()}/api/app/v${serverVer()}';
  }
}
