import 'package:fake_http/fake_http.dart';
import 'package:flutter/foundation.dart';
import 'package:meta/meta.dart';
import 'package:star_xms/star_xms.dart';

import 'domain/login_data.dart';
import 'domain/matter_data.dart';

class StarAdminRepository extends XmsRepository {
  StarAdminRepository({
    @required String serverUrl,
    @required String appId,
    @required String appSecret,
    @required AsyncValueGetter<String> token,
    @required AsyncValueGetter<String> uuid,
    @required AsyncValueGetter<String> userAgent,
    @required AsyncValueGetter<Directory> cacheDir,
    @required AsyncValueGetter<bool> connectivity,
    CookieJar cookieJar,
    LoggerLevel loggerLevel = LoggerLevel.BASIC,
    LoggerFactory loggerFactory = LoggerFactory.PLATFORM,
  }) : super(
          serverUrl: serverUrl,
          appId: appId,
          appSecret: appSecret,
          token: token,
          uuid: uuid,
          userAgent: userAgent,
          cacheDir: cacheDir,
          connectivity: connectivity,
          cookieJar: cookieJar,
          loggerLevel: loggerLevel,
          loggerFactory: loggerFactory,
        );

  /// 登录
  Future<ResultEntity<LoginData>> login(
    String phone,
    String password,
  ) {
    assert(phone != null && phone.isNotEmpty);
    assert(password != null && password.isNotEmpty);
    return client
        .newCall(RequestBuilder()
            .url(HttpUrl.parse('$serverUrl/login'))
            .post(RequestBody.textBody(
                MediaType.JSON,
                json.encode(<String, dynamic>{
                  'phone': phone,
                  'password': hex
                      .encode(md5.convert(utf8.encode(password)).bytes)
                      .toLowerCase()
                })))
            .build())
        .enqueue()
        .then((Response response) {
      return resultEntityAsMap(response, LoginDataSerializer());
    });
  }

  /// 我的事项
  Future<PageEntity<MatterData>> matters(
    int processType,
    int pageStart, {
    bool refresh = false,
  }) {
    return client
        .newCall(RequestBuilder()
            .url(HttpUrl.parse('$serverUrl/mine/matters')
                .newBuilder()
                .addQueryParameter('processType', '$processType')
                .addQueryParameter('pageStart', '$pageStart')
                .build())
            .get()
            .cacheControl(cacheControl(refresh))
            .build())
        .enqueue()
        .then((Response response) {
      return pageEntityAsMap(response, MatterDataSerializer());
    });
  }
}
