import 'package:fake_http/fake_http.dart';
import 'package:flutter_test/flutter_test.dart';
import 'package:star_xms/star_xms.dart';

import 'src/domain/login_data.dart';
import 'src/domain/matter_data.dart';
import 'src/server/server.dart';
import 'src/star_admin_repository.dart';
import 'src/util/token_persistor.dart';

void main() {
  FileSystem fileSystem = const LocalFileSystem();

  Directory directory = fileSystem.currentDirectory
      .childDirectory('build')
      .childDirectory('star_admin');
  if (!directory.existsSync()) {
    directory.createSync(recursive: true);
  }

  Server server = Server.PROD;
  TokenPersistor persistor = TokenPersistor.MEMORY;

  StarAdminRepository repository = StarAdminRepository(
    serverUrl: server.serverApiUrl(),
    appId: server.appId,
    appSecret: server.appSecret,
    token: () => persistor.loadToken(),
    uuid: () => Future<String>.value('xxx'),
    userAgent: () => Future<String>.value(Version.userAgent()),
    cacheDir: () => Future<Directory>.value(directory),
    connectivity: () => Future<bool>.value(true),
    loggerLevel: LoggerLevel.BODY,
  );

  test('smoke test - login', () async {
    await repository
        .login('15306978908', '123456')
        .then((ResultEntity<LoginData> entity) {
      print('${entity.status} - ${entity.msg}');
      if (entity.isOK()) {
        LoginData loginData = entity.result;
        if (loginData != null &&
            loginData.userID != null &&
            loginData.token != null) {
          persistor.saveToken(loginData.userID, loginData.token);
        }
      }
    });
  });

  test('smoke test - matters', () async {
    await repository
        .matters(MatterData.PROCESS_TYPE_DO, Page.PAGE_START)
        .then((PageEntity<MatterData> entity) {
      print('${entity.status} - ${entity.msg}');
      if (entity.isOK()) {
        if (entity.page != null &&
            entity.result != null &&
            entity.result.isNotEmpty) {
          entity.result.forEach((MatterData data) {
            print('title: ${data.title}');
          });
        }
      }
    });
  });
}
